@@ -41,8 +41,6 @@ class MediaStream(Base):
      
         self.debug_filename = None 
 
-
-
         if options.get('delay'):
             g.logger.Debug(1, 'Waiting for {} seconds'.format(options.get('delay')))
             time.sleep(int(options.get('delay')))
@@ -101,9 +99,11 @@ class MediaStream(Base):
                 g.logger.Debug(2,'Using URL {} for stream'.format(stream))
                 self.type = 'image'
         
-        
         if self.options.get('frame_set'):
-            self.frame_set = self.options.get('frame_set').split(',')
+            if isinstance(self.options.get('frame_set'),str):
+                self.frame_set = self.options.get('frame_set').split(',')
+            else:
+                self.frame_set = [ str(i) for i in self.options.get('frame_set')]
             if 'alarm' in self.frame_set or 'snapshot' in self.frame_set:
                 if not self.api or self.type == 'video':
                     # if we are not using ZM indirection, we cannot use 'alarm' 'snapshot' etc.
@@ -176,6 +176,10 @@ class MediaStream(Base):
                 g.logger.Error (f'Could not delete file(s):{e}')
 
     def read(self):
+        if self.options.get('delay_between_frames') and self.frames_processed > 0:
+            g.logger.Debug(4,'Sleeping {} seconds before reading next frame'.format(self.options.get('delay_between_frames')))
+            time.sleep(int(self.options.get('delay_between_frames')))
+
         if (self.type == 'file'):
             frame = cv2.imread(self.stream)
             self.last_frame_id_read = 1
@@ -244,15 +248,19 @@ class MediaStream(Base):
                     self.next_frameid_to_read = 0
                     return None 
                 if self.frame_set[self.next_frame_set_index]=='snapshot' and self.api:
-                    g.logger.Debug(4,'Trying to convert snapshot to a real frame id')
-                    try:
-                        eurl = '{}/events/{}.json'.format(self.api.get_apibase(),self.eid)
-                        res = self.api._make_request(eurl)
-                        fid = res.get('event',{}).get('Event',{}).get('MaxScoreFrameId')
-                        g.logger.Debug(4,'At the point of analysis, Event:{} snapshot frame id was:{},so using it'.format(self.eid, fid))
-                        self.frame_set[self.next_frame_set_index]= str(fid)
-                    except Exception as e:
-                        g.logger.Debug(4,' Failed retrieving snapshot frame ID:{}'.format(e))
+                    if self.options.get('delay_between_snapshots') and self.frames_processed > 0:
+                        g.logger.Debug(4,'Sleeping {} seconds before reading next snapshot frame'.format(self.options.get('delay_between_snapshots')))
+                        time.sleep(int(self.options.get('delay_between_snapshots')))
+                    if self.options.get('convert_snapshot_to_fid'):
+                        g.logger.Debug(4,'Trying to convert snapshot to a real frame id')
+                        try:
+                            eurl = '{}/events/{}.json'.format(self.api.get_apibase(),self.eid)
+                            res = self.api._make_request(eurl)
+                            fid = res.get('event',{}).get('Event',{}).get('MaxScoreFrameId')
+                            g.logger.Debug(4,'At the point of analysis, Event:{} snapshot frame id was:{},so using it'.format(self.eid, fid))
+                            self.frame_set[self.next_frame_set_index]= str(fid)
+                        except Exception as e:
+                            g.logger.Debug(4,' Failed retrieving snapshot frame ID:{}'.format(e))
 
                 url = '{}&fid={}'.format(self.stream,self.frame_set[self.next_frame_set_index])
 
