@@ -219,18 +219,20 @@ class DetectSequence(Base):
     # once all bounding boxes are detected, we check to see if any of them
     # intersect the polygons, if specified
     # it also makes sure only patterns specified in detect_pattern are drawn
-    def _process_past_detections(self,m,bbox, label, conf, mid, model_name):
+    def _process_past_detections(self,bbox, label, conf, matched_detection_types, matched_model_names):
 
         try:
             FileNotFoundError
         except NameError:
             FileNotFoundError = IOError
 
+        mid = self.stream_options.get('mid')
         if not mid:
             g.logger.Debug(1,
                 'Monitor ID not specified, cannot match past detections')
             return bbox, label, conf
-        image_path = self.global_config.get('image_path') or m.get_options().get('image_path')
+
+        image_path = self.global_config.get('image_path') or self.ml_options.get('general',{}).get('image_path') 
         mon_file = '{}/monitor-{}-data.pkl'.format(image_path,mid)
         g.logger.Debug(2,'trying to load ' + mon_file)
         saved_bs=[]
@@ -244,7 +246,7 @@ class DetectSequence(Base):
             fh.close()
         except FileNotFoundError:
             g.logger.Debug(1,'No history data file found for monitor {}'.format(mid))
-            return bbox, label, conf
+            #return bbox, label, conf
         except EOFError:
             g.logger.Debug(1,'Empty file found for monitor {}'.format(mid))
             g.logger.Debug (1,'Going to remove {}'.format(mon_file))
@@ -279,19 +281,31 @@ class DetectSequence(Base):
                 global_max_diff_area=5
        
         #g.logger.Debug (1,'loaded past: bbox={}, labels={}'.format(saved_bs, saved_ls));
-        g.logger.Debug (4, 'process_past_detections: Global use_percent:{}, max_diff_area:{}'.format(global_use_percent,global_max_diff_area))
+        g.logger.Debug (4, 'Globals:past detection:use_percent:{}, max_diff_area:{}'.format(global_use_percent,global_max_diff_area))
         new_label = []
         new_bbox = []
         new_conf = []
+        new_detection_types = []
+        new_model_names = []
 
         for idx, b in enumerate(bbox):
 
+            if label[idx] in self.ml_options.get('general',{}).get('ignore_past_detection_labels',[]):
+                g.logger.Debug (4, '{} is in ignore list for past detection match, skipping'.format(label[idx]))
+                new_bbox.append(b)
+                new_label.append(label[idx])
+                new_conf.append(conf[idx])
+                new_detection_types.append(matched_detection_types[idx])
+                new_model_names.append(matched_model_names[idx])
+                continue
+
             max_diff_area = global_max_diff_area
             use_percent = global_use_percent
 
             label_max_diff_area= '{}_past_det_max_diff_area'.format(label[idx])
-            mda = m.get_options().get(label_max_diff_area) or self.global_config.get(label_max_diff_area)
-            if mda:
+            mda = self.ml_options.get('general',{}).get(label_max_diff_area) or self.global_config.get(label_max_diff_area)
+            # handle the case where you don't have a label_ substitution for a monitor
+            if mda and not mda.startswith('{{'):
                 g.logger.Debug(4, 'Found {}={}'.format(label_max_diff_area, mda))
                 _m = re.match('(\d+)(px|%)?$',mda,re.IGNORECASE)
                 if _m:
@@ -328,7 +342,7 @@ class DetectSequence(Base):
                 saved_b.insert(3, (saved_b[0][0], saved_b[2][1]))
                 saved_obj = Polygon(saved_b)
                 max_diff_pixels = max_diff_area
-
+                g.logger.Debug (4, 'match_past_detections: Comparing  saved {}@{} to {}@{}'.format(saved_ls[saved_idx], saved_b, label[idx],b))
                 if saved_obj.intersects(obj):
                     if obj.contains(saved_obj):
                         diff_area = obj.difference(saved_obj).area
@@ -352,26 +366,35 @@ class DetectSequence(Base):
                 new_bbox.append(old_b)
                 new_label.append(label[idx])
                 new_conf.append(conf[idx])
+                new_detection_types.append(matched_detection_types[idx])
+                new_model_names.append(matched_model_names[idx])
 
-        return new_bbox, new_label, new_conf
+          # save current objects for future comparisons
+        g.logger.Debug(1,
+            'Saving detections for monitor {} for future match'.format(
+                self.stream_options.get('mid')))
+        try:
+            f = open(mon_file, "wb")
+            pickle.dump(bbox, f)
+            pickle.dump(label, f)
+            pickle.dump(conf, f)
+            g.logger.Debug(4, 'saving boxes:{}, labels:{} confs:{} to {}'.format(bbox,label,conf, mon_file))
+            f.close()
+        except Exception as e:
+            g.logger.Error(f'Error writing to {mon_file}, past detections not recorded:{e}')
+        return new_bbox, new_label, new_conf, new_detection_types, new_model_names
 
 
-    def _filter_detections(self,m, seq, box,label,conf, polygons, h,w):
+    def _filter_detections(self, seq, box,label,conf, polygons, h,w, model_names):
         
-        # show_prefix
-        stripped_labels = label[:]
-        for idx, l in enumerate(stripped_labels):
-            if l.startswith('('):
-                items = l.split(') ')
-                stripped_labels[idx] = items[1] if len(items) == 2 else items[0]
-
+     
         # remember this needs to occur after a frame
         # is read, otherwise we don't have dimensions
 
         #print ("************ POLY={}".format(polygons))
 
         global_max_object_area = 0
-        mds=m.get_options().get('max_detection_size') or self.global_config.get('max_detection_size')
+        mds= self.ml_options.get('general',{}).get('max_detection_size') or self.global_config.get('max_detection_size')
         if mds:  
                 g.logger.Debug(3,'Max object size found to be: {}'.format(mds))
                 # Let's make sure its the right size
@@ -416,11 +439,13 @@ class DetectSequence(Base):
         new_bbox =[]
         new_conf = []
         new_err = []
+        new_model_names = []
         for idx,b in enumerate(box):
             max_object_area = global_max_object_area
             label_max_object_area= '{}_max_detection_size'.format(label[idx])
-            moa = m.get_options().get(label_max_object_area) or self.global_config.get(label_max_object_area)
-            if moa:
+            moa =  self.ml_options.get('general',{}).get(label_max_object_area) or self.global_config.get(label_max_object_area)
+            # Handle case where you don't have a label_ substitution for a monitor
+            if moa and not moa.startswith('{{'):
                 g.logger.Debug(4,'Found {}={}'.format(label_max_object_area,moa))
                 # Let's make sure its the right size
                 _m = re.match('(\d*\.?\d*)(px|%)?$', moa,
@@ -453,7 +478,7 @@ class DetectSequence(Base):
                     if  p['pattern']:
                         g.logger.Debug(3, '{} polygon/zone has its own pattern of {}, using that'.format(p['name'],p['pattern']))
                         r = re.compile(p['pattern'])
-                        match = list(filter(r.match, stripped_labels))
+                        match = list(filter(r.match, label))
                     else:
                         '''
                         g.logger.Debug (1,'**********************************')
@@ -469,15 +494,16 @@ class DetectSequence(Base):
 
                         r = re.compile(match_pattern)
 
-                        match = list(filter(r.match, stripped_labels))
+                        match = list(filter(r.match, label))
 
                     #if label[idx].startswith('face:') or label[idx].startswith('alpr:') or label[idx] in match:
-                    if stripped_labels[idx] in match:
+                    if label[idx] in match:
                         g.logger.Debug(3,'{} intersects object:{}[{}]'.format(
                             p['name'], label[idx], b))
                         new_label.append(label[idx])
                         new_bbox.append(old_b)
                         new_conf.append(conf[idx])
+                        new_model_names.append(model_names[idx])
                     else:
                         new_err.append(old_b)
 
@@ -494,7 +520,7 @@ class DetectSequence(Base):
                     format(label[idx], obj))
         # out of primary bbox loop
         #print ("NEW ERR IS {}".format(new_err))
-        return new_bbox, new_label, new_conf, new_err
+        return new_bbox, new_label, new_conf, new_err, new_model_names
 
 
     def detect_stream(self, stream, options={}, ml_overrides={}):
@@ -506,13 +532,15 @@ class DetectSequence(Base):
             options (dict, optional): Various options that control the detection process. Defaults to {}:
             
                 - delay (int): Delay in seconds before starting media stream
+                - delay_between_frames (int): Delay in seconds between each frame read 
+                - delay_between_snapshots (int): Delay in seconds between each snapshot frame read (useful if you want to read snapshot multiple times, for example. frameset: ['snapshot','snapshot','snapshot'])
                 - download (boolean): if True, will download video before analysis. Defaults to False
                 - download_dir (string): directory where downloads will be kept (only applies to videos). Default is /tmp
                 - start_frame (int): Which frame to start analysis. Default 1.
                 - frame_skip: (int): Number of frames to skip in video (example, 3 means process every 3rd frame)
                 - max_frames (int): Total number of frames to process before stopping
                 - pattern (string): regexp for objects that will be matched. 'frame_strategy' key below will be applied to only objects that match this pattern
-                - frame_set (string): comma separated frames to read. Example 'alarm,21,31,41,snapshot'
+                - frame_set (string or list): comma separated frames to read. Example 'alarm,21,31,41,snapshot' or ['snapshot','alarm','1','2']
                   Note that if you are specifying frame IDs and using ZM, remember that ZM has a frame buffer
                   Default is 20, I think. So you may want to start at frame 21.
                 - contig_frames_before_error (int): How many contiguous frames should fail before we give up on reading this stream. Default 5
@@ -530,6 +558,9 @@ class DetectSequence(Base):
            
                 - resize (int): Width to resize image, default 800
                 - polygons(object): object # set of polygons that the detected image needs to intersect
+                - convert_snapshot_to_fid (bool): if True, will convert 'snapshot' to an actual fid. If you are seeing 
+                  boxes at wrong places for snapshot frames, this may fix it. However, it can also result in frame 404 errors 
+                  if that frame ID is not yet written to disk. So you may want to add a delay if you enable this. Default is False.
                 
         Returns:
            - object: representing matched frame, consists of:
@@ -563,9 +594,10 @@ class DetectSequence(Base):
         matched_e = []
         matched_l = []
         matched_c = []
-        matched_models = []
+        matched_detection_types = [] # object, face, etc
         matched_frame_id = None
         matched_images=[]
+        matched_model_names = [] # coral, yolo etc
         
         matched_frame_img = None
         manual_locking = False
@@ -585,7 +617,6 @@ class DetectSequence(Base):
 
         polygons = copy.copy(self.stream_options.get('polygons',[]))
 
-        
         # Loops across all frames
         while self.media.more():
             frame = self.media.read()
@@ -600,7 +631,8 @@ class DetectSequence(Base):
             _boxes_in_frame = []
             _error_boxes_in_frame = []
             _confs_in_frame = []
-            _models_in_frame = []
+            _detection_types_in_frame = []
+            _model_names_in_frame = []
 
             # For each frame, loop across all models
             found = False
@@ -635,14 +667,21 @@ class DetectSequence(Base):
                 _l_best_in_same_model = [] 
                 _c_best_in_same_model = []
                 _e_best_in_same_model = []
+                _m_best_in_same_model = []
 
                 cnt = 1
                 # For each model, loop across different variations
                 for m in self.models[seq]:
                     g.logger.Debug(3,'--------- Frame:{} Running variation: #{} -------------'.format(self.media.get_last_read_frame(),cnt))
                     cnt +=1
+                    pre_existing_labels = m.get_options().get('pre_existing_labels')
+                    if pre_existing_labels:
+                        g.logger.Debug(2,'Making sure we have matched one of {} in {} before we proceed'.format(pre_existing_labels, _l_best_in_same_model))
+                        if not any(x in _l_best_in_same_model for x in pre_existing_labels):
+                            g.logger.Debug(1,'Did not find pre existing labels, not running sequence in model')
+                            continue
                     try:
-                        _b,_l,_c = m.detect(image=frame)
+                        _b,_l,_c,_m = m.detect(image=frame)
                         g.logger.Debug(4,'This model iteration inside {} found: labels: {},conf:{}'.format(seq, _l, _c))
                     except Exception as e:
                         g.logger.Error ('Error running model: {}'.format(e))
@@ -651,44 +690,14 @@ class DetectSequence(Base):
 
                     # Now let's make sure the labels match our pattern
                     h,w,_ = frame.shape
-                    _b,_l,_c, _e = self._filter_detections(m, seq,_b,_l,_c, polygons, h, w)
+                    _b,_l,_c, _e, _m = self._filter_detections(seq,_b,_l,_c, polygons, h, w, _m)
                     if _e:
                         _e_best_in_same_model.extend(_e)
                     if not len(_l):
                         continue
 
-                    # Now let's take past detections into consideration 
-                    # let's remove past detections first, if enabled 
-                
-                    #g.logger.Debug(4,'REMOVE ME: Self m options: {}'.format(m.get_options()))
-                    mpd = m.get_options().get('match_past_detections') or self.global_config.get('match_past_detection')
-                    if mpd == 'yes' and self.stream_options.get('mid'):
-                        # point detections to post processed data set
-                        g.logger.Info('Removing matches to past detections for monitor:{}'.format(self.stream_options.get('mid')))
-                        #g.logger.Debug (1,'BEFORE PAST: {}, {}, {}'.format(_b,_l,_c))
-
-                        _b,_l,_c = self._process_past_detections(m,_b, _l, _c, self.stream_options.get('mid'),seq)
-                        #g.logger.Debug (1,'AFTER PAST: {}, {}, {}'.format(_b,_l,_c))
-                        if not len (_l):
-                            continue
-                        
-                        # save current objects for future comparisons
-                        if seq!='face':
-                            g.logger.Debug(1,
-                                'Saving detections for monitor {} for future match'.format(
-                                    self.stream_options.get('mid')))
-                            try:
-                                image_path = self.global_config.get('image_path') or m.get_options().get('image_path')
-                                mon_file = '{}/monitor-{}-data.pkl'.format(image_path,self.stream_options.get('mid'))
-                                f = open(mon_file, "wb")
-                                pickle.dump(_b, f)
-                                pickle.dump(_l, f)
-                                pickle.dump(_c, f)
-                                f.close()
-                            except Exception as e:
-                                g.logger.Error(f'Error writing to {mon_file}, past detections not recorded:{e}')
-
-
+                   
+                      
                     if  ((same_model_sequence_strategy == 'first') 
                     or ((same_model_sequence_strategy == 'most') and (len(_l) > len(_l_best_in_same_model))) 
                     or ((same_model_sequence_strategy == 'most_unique') and (len(set(_l)) > len(set(_l_best_in_same_model))))):
@@ -696,11 +705,15 @@ class DetectSequence(Base):
                         _l_best_in_same_model = _l
                         _c_best_in_same_model = _c
                         _e_best_in_same_model = _e
+                        _m_best_in_same_model = _m
+
                     elif same_model_sequence_strategy == 'union':
                         _b_best_in_same_model.extend(_b)
                         _l_best_in_same_model.extend(_l)
                         _c_best_in_same_model.extend(_c)
                         _e_best_in_same_model.extend(_e)
+                        _m_best_in_same_model.extend(_m)
+
                     if _l_best_in_same_model and self.stream_options.get('save_analyzed_frames') and self.media.get_debug_filename():
                             d = self.stream_options.get('save_frames_dir','/tmp')
                             f = '{}/{}-analyzed-{}.jpg'.format(d,self.media.get_debug_filename(), media.get_last_read_frame())
@@ -722,7 +735,8 @@ class DetectSequence(Base):
                     _boxes_in_frame.extend(_b_best_in_same_model)
                     _confs_in_frame.extend(_c_best_in_same_model)
                     _error_boxes_in_frame.extend(_e_best_in_same_model)
-                    _models_in_frame.append(seq)
+                    _detection_types_in_frame.extend([seq]*len(_l_best_in_same_model))
+                    _model_names_in_frame.extend(_m_best_in_same_model)
                     if (frame_strategy == 'first'):
                         g.logger.Debug (2, 'Breaking out of main model loop as strategy is first')
                         break
@@ -738,7 +752,9 @@ class DetectSequence(Base):
                         'error_boxes': _error_boxes_in_frame,
                         'labels': _labels_in_frame,
                         'confidences': _confs_in_frame,
-                        'models': _models_in_frame
+                        'detection_types': _detection_types_in_frame,
+                        'model_names': _model_names_in_frame
+                        
                         
                     }
                 )
@@ -754,14 +770,15 @@ class DetectSequence(Base):
         for idx,item in enumerate(all_matches):
             if  ((frame_strategy == 'first') or 
             ((frame_strategy == 'most') and (len(item['labels']) > len(matched_l))) or
-            ((frame_strategy == 'most_models') and (len(item['models']) > len(matched_models))) or
+            ((frame_strategy == 'most_models') and (len(item['detection_types']) > len(matched_detection_types))) or
             ((frame_strategy == 'most_unique') and (len(set(item['labels'])) > len(set(matched_l))))):
                 matched_b =item['boxes']
                 matched_e = item['error_boxes']
                 matched_c = item['confidences']
                 matched_l  = item['labels']            
                 matched_frame_id = item['frame_id']
-                matched_models = item['models']
+                matched_detection_types = item['detection_types']
+                matched_model_names = item['model_names']
                 matched_frame_img = matched_images[idx]
        
         if manual_locking:
@@ -769,6 +786,19 @@ class DetectSequence(Base):
                 for m in self.models[seq]:
                     m.release_lock()
 
+        
+        # Now let's take past detections into consideration 
+        # let's remove past detections first, if enabled 
+        #g.logger.Debug (4, 'REMOVE ME: {}'.format(self.ml_options.get('general')))        
+        mpd = self.ml_options.get('general',{}).get('match_past_detections') or self.global_config.get('match_past_detections')
+        if mpd == 'yes' and self.stream_options.get('mid'):
+            # point detections to post processed data set
+            g.logger.Info('Removing matches to past detections for monitor:{}'.format(self.stream_options.get('mid')))
+            #g.logger.Debug (1,'REMOVE ME BEFORE PAST: b:{}, l:{}, c:{}, dt:{}, mn:{}'.format(matched_b,matched_l,matched_c, matched_detection_types, matched_model_names))
+
+            matched_b,matched_l,matched_c, matched_detection_types, matched_model_names = self._process_past_detections(matched_b, matched_l, matched_c, matched_detection_types, matched_model_names)
+            #g.logger.Debug (1,'REMOVE ME AFTER PAST: b:{}, l:{}, c:{}, dt:{}, mn:{}'.format(matched_b,matched_l,matched_c, matched_detection_types, matched_model_names))
+
         diff_time = t.stop_and_get_ms()
 
         g.logger.Debug(
@@ -781,6 +811,7 @@ class DetectSequence(Base):
             'labels': matched_l,
             'confidences': matched_c,
             'frame_id': matched_frame_id,
+            'model_names': matched_model_names,
             'image_dimensions': self.media.image_dimensions(),
             #'type': matched_type,
             'image': matched_frame_img,
